package com.galaxis.manatee.service;

import com.galaxis.manatee.dao.ChuanyunUserCompanyDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunUserCompanyDO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.time.Instant;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author zcj
 * @version 0.1
 * @date 2021/3/11 3:30 上午
 */
@Slf4j
@Service
public class LogService {

    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final LogStandardService logStandardService;
    private final LogUpdateService logUpdateService;

    public LogService(ChuanyunUserCompanyDao chuanyunUserCompanyDao, LogStandardService logStandardService, LogUpdateService logUpdateService) {
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.logStandardService = logStandardService;
        this.logUpdateService = logUpdateService;
    }

    /**
     * 标准化所有周工时信息
     */
    public void standardAllWeekMemberHour() {
        long start = Instant.now().getEpochSecond();
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        userIdList.stream().distinct().collect(Collectors.toList()).parallelStream().forEach(logStandardService::standardWeekMemberHourByUserId);
        log.info("标准化所有周工时耗时" + (Instant.now().getEpochSecond() - start) + "秒");
    }

    /**
     * 标准化一年内周工时信息
     */
    @Async("threadPoolTaskExecutor")
    public void standardOneYearWeekMemberHour() {
        long start = Instant.now().getEpochSecond();
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        userIdList.stream().distinct().collect(Collectors.toList()).parallelStream().forEach(logStandardService::standardWeekMemberHourByUserId);
        log.info("标准化所有周工时耗时" + (Instant.now().getEpochSecond() - start) + "秒");
    }

    /**
     * 标准化所有月工时
     */
    public void standardAllSelfCost() {
        long start = Instant.now().getEpochSecond();
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        userIdList.stream().distinct().collect(Collectors.toList()).parallelStream().forEach(logStandardService::standardSelfCostByUserId);
        log.info("标准化所有月工时耗时" + (Instant.now().getEpochSecond() - start) + "秒");
    }

    /**
     * 标准化一年内月工时
     */
    @Async("threadPoolTaskExecutor")
    public void standardOneYearSelfCost() {
        long start = Instant.now().getEpochSecond();
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        userIdList.stream().distinct().collect(Collectors.toList()).parallelStream().forEach(logStandardService::standardSelfCostByUserId);
        log.info("标准化所有月工时耗时 {} ", Instant.now().getEpochSecond() - start);
    }

    /**
     * 更新所有日工时
     */
    public void updateAllChuanyunSelfWorkHour() {
        long startTime = Instant.now().getEpochSecond();
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        userIdList.stream().distinct().collect(Collectors.toList()).parallelStream().forEach(logUpdateService::updateChuanyunSelfWorkHourListByUserId);
        log.info("更新所有每日工时花费" + (Instant.now().getEpochSecond() - startTime) + "秒");
    }

    /**
     * 更新半年内日工时
     */
    public void updateHalfYearChuanyunSelfWorkHour() {
        long startTime = Instant.now().getEpochSecond();
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        userIdList.stream().distinct().collect(Collectors.toList()).parallelStream().forEach(logUpdateService::pushHalfYearDayWorkHour);
        log.info("更新所有每日工时花费 {} 秒", Instant.now().getEpochSecond() - startTime);
    }

    /**
     * 更新所有周工时
     */
    public void updateAllChuanyunWeekMemberHour() {
        long startTime = Instant.now().getEpochSecond();
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        userIdList.stream().distinct().collect(Collectors.toList()).parallelStream().forEach(logUpdateService::updateChuanyunWeekMemberHourListByUserId);
        log.info("更新所有每周工时花费" + (Instant.now().getEpochSecond() - startTime) + "秒");
    }



    /**
     * 更新所有月工时
     */
    public void updateAllChuanyunSelfCost() {
        long startTime = Instant.now().getEpochSecond();
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        userIdList.stream().distinct().collect(Collectors.toList()).parallelStream().forEach(logUpdateService::updateChuanyunSelfCostListByUserId);
        log.info("更新所有每月工时花费" + (Instant.now().getEpochSecond() - startTime) + "秒");
    }

    /**
     * 更新去年至今周工时
     */
    public void pushLastYearToPresentWeekMemberHour() {
        long startTime = Instant.now().getEpochSecond();
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        userIdList.stream().distinct().collect(Collectors.toList()).parallelStream().forEach(logUpdateService::pushLastYearToPresentWeekMemberHourListByUserId);
        log.info("更新一年内每周工时花费" + (Instant.now().getEpochSecond() - startTime) + "秒");
    }

    /**
     * 更新一年内月工时
     */
    public void pushOneYearChuanyunSelfCost() {
        long startTime = Instant.now().getEpochSecond();
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        userIdList.stream().distinct().collect(Collectors.toList()).parallelStream().forEach(logUpdateService::pushOneYearSelfCostListByUserId);
        log.info("更新所有每月工时花费" + (Instant.now().getEpochSecond() - startTime) + "秒");
    }
}
